-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- DISCLAIMER - Execute at your own risk. Read this script and thoroughly test first!!!
--
-- Written by Kimberly L. Tripp of SYSolutions, Inc.
-- For more code samples go to http://www.sqlskills.com
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

USE CREDIT
go

--Indexed Views vs performing the aggregate each time!
-- ONLY WORKS IN ENTERPRISE EDITION (or Developer)

-- CHECK out the statistics io and the time AND turn on 
-- graphical showplan with Ctrl+K
-- DBCC FREEPROCCACHE --(to clear plans from cache)
-- DBCC DROPCLEANBUFFERS --(to clear data from cache)

/*
SET STATISTICS IO ON
DECLARE @StartTime	datetime, @EndTime	datetime
SELECT @StartTime = getdate()
-- PUT COMMAND HERE
SELECT @EndTime = getdate()
SELECT 'TOTAL TIME (ms)' = datediff (ms, @StartTime, @EndTime)
*/

-- Keep track of time here = ????ms
-- Keep track of i/o here = ???i/o

-- Finally you can compare all 4...but a bunch of hints are needed to get
-- SQL Server to avoid using the fast/efficient index on the view...

--Here's the final comparisons query
SET STATISTICS IO ON
go
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge (index = 0) -- table scan
GROUP BY member_no
OPTION (EXPAND VIEWS) -- this is to force sql server to go to the base table
go

--CREATE INDEX Covering1 on charge(charge_amt, member_no) -- Not in order by the Group By 
-- This is NOT ideal but it's better than the Table Scan
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge (index = Covering1)
GROUP BY member_no
OPTION (EXPAND VIEWS)
go

--CREATE INDEX Covering2 on charge(member_no, charge_amt) -- In Order by the Group By 

SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge (index = Covering2)
GROUP BY member_no
OPTION (EXPAND VIEWS)
go

-- This is the definition of the VIEW.
-- CREATE VIEW dbo.SumOfAllChargesByMember
-- 	WITH SCHEMABINDING  -- required if you want indexes!
-- AS
-- SELECT member_no AS MemberNo, 
-- 	count_big(*) AS NumberOfCharges,
-- 	sum(charge_amt) AS TotalSales
-- FROM dbo.charge
-- GROUP BY member_no
go
-- This is the Index Created on the view. Notice it is a UNIQUE Clustered.
-- This is a requirement of Indexed Views.
-- CREATE UNIQUE CLUSTERED INDEX SumofAllChargesIndex
-- 	ON dbo.SumOfAllChargesByMember (MemberNo) 

-- Notice NO hints have to be used! This is most certainly the ideal
-- plan! 
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge
GROUP BY member_no
go

-- And NOW other queries the can benefit from the index on the view
-- Check out the plan for the following query!

-- SELECT member_no as MemberNo, 
-- 	avg(charge_amt) AS TotalSales
-- FROM dbo.charge
-- GROUP BY member_no
go
